precision lowp int; 
precision lowp float;

attribute lowp vec2 IN_uv;
attribute lowp vec4 IN_vertexcolour;

uniform highp mat4 worldmatrix;
uniform highp mat3 worldmatrixrot;
uniform highp mat4 viewmatrix;
uniform highp mat4 projectionmatrix;

uniform mediump vec4 displaycolour;  //0..255

varying lowp vec4 uvs;
varying lowp vec4 vertexcolour;
varying lowp vec3 worldnormal;

varying highp vec3 worldpos;


uniform lowp vec3 IN_scene_eye_dir;
uniform mediump vec3 IN_scene_eye_pos;

uniform lowp vec4  IN_material_colour;

#if defined( USE_SKINNING)

attribute mediump vec4  IN_bone_pos0;  //automatic upcast to vec4 with w=1
attribute mediump vec4  IN_bone_pos1;
attribute mediump vec4  IN_bone_pos2;
attribute mediump vec3  IN_bone_norm0;
attribute mediump vec3  IN_bone_norm1;
attribute mediump vec3  IN_bone_norm2;

attribute lowp vec3 IN_boneWeights;
attribute mediump float IN_boneCount;
attribute mediump vec3 IN_boneIndexs;

uniform mediump mat4 IN_bone_mats[ MAX_BONES_TOTAL ];

#else

attribute mediump vec4 IN_position;
attribute lowp vec3 IN_normal;

#endif 


//////



void main()
{
	uvs=vec4(IN_uv.x,IN_uv.y,0.0,0.0);
	
	vertexcolour=IN_vertexcolour * IN_material_colour  * ( displaycolour / 255.0 );
	mediump mat4 modelView = mat4(viewmatrix * worldmatrix);
		
#if defined( USE_SKINNING)


	mediump vec4 vertexPos;
	if ( IN_boneCount <= 1.0 )
	{
		vertexPos= (( IN_bone_mats[ int( IN_boneIndexs[0]) ] * IN_bone_pos0) * IN_boneWeights[0]);

		//Create useful matrixes from the scaled ones
		//These normals are going to be wrong once you start doing non-uniform scales, but honestly the error is pretty low
		mediump mat3 matrix0 =mat3( IN_bone_mats[ int( IN_boneIndexs[0]) ]);

		lowp vec3 normal =   (( matrix0 * IN_bone_norm0) * IN_boneWeights[0]);
		
		worldnormal=  ( worldmatrixrot * normalize( normal )); //This normalize fixes alot of the drama
		worldpos = (worldmatrix * vertexPos ).xyz;
	}
	else
	if ( IN_boneCount == 2.0 )
	{
		vertexPos= (( IN_bone_mats[ int( IN_boneIndexs[0]) ] * IN_bone_pos0) * IN_boneWeights[0]) +
						(( IN_bone_mats[ int( IN_boneIndexs[1]) ] * IN_bone_pos1) * IN_boneWeights[1]);
		
		//Create useful matrixes from the scaled ones
		//These normals are going to be wrong once you start doing non-uniform scales, but honestly the error is pretty low
		mediump mat3 matrix0 =mat3( IN_bone_mats[ int( IN_boneIndexs[0]) ]);
		mediump mat3 matrix1 =mat3( IN_bone_mats[ int( IN_boneIndexs[1]) ]);

		lowp vec3 normal =   (( matrix0 * IN_bone_norm0) * IN_boneWeights[0]) +
						(( matrix1 * IN_bone_norm1) * IN_boneWeights[1]);
						
		worldnormal=  ( worldmatrixrot * normalize( normal )); //This normalize fixes alot of the drama
		worldpos = (worldmatrix * vertexPos ).xyz;
	}
	else
	{
		vertexPos= (( IN_bone_mats[ int( IN_boneIndexs[0]) ] * IN_bone_pos0) * IN_boneWeights[0]) +
						(( IN_bone_mats[ int( IN_boneIndexs[1]) ] * IN_bone_pos1) * IN_boneWeights[1]) +
						(( IN_bone_mats[ int( IN_boneIndexs[2]) ] * IN_bone_pos2) * IN_boneWeights[2]);

		//Create useful matrixes from the scaled ones
		//These normals are going to be wrong once you start doing non-uniform scales, but honestly the error is pretty low
		mediump mat3 matrix0 =mat3( IN_bone_mats[ int( IN_boneIndexs[0]) ]);
		mediump mat3 matrix1 =mat3( IN_bone_mats[ int( IN_boneIndexs[1]) ]);
		mediump mat3 matrix2 =mat3( IN_bone_mats[ int( IN_boneIndexs[2]) ]);

		lowp vec3 normal =   (( matrix0 * IN_bone_norm0) * IN_boneWeights[0]) +
						(( matrix1 * IN_bone_norm1) * IN_boneWeights[1]) +
						(( matrix2 * IN_bone_norm2) * IN_boneWeights[2]);
	
		worldnormal=  ( worldmatrixrot * normalize( normal )); //This normalize fixes alot of the drama
		worldpos = (worldmatrix * vertexPos ).xyz;
	}

#else

	mediump vec4 vertexPos = IN_position;
	worldnormal = worldmatrixrot * normalize(IN_normal) ; 
	worldpos = (worldmatrix * vertexPos).xyz;
#endif

	gl_Position = (projectionmatrix *modelView ) *  vertexPos;
}

